﻿using System.Collections.Concurrent;
using System.Diagnostics;
using System.Text.RegularExpressions;
using Discord;
using Discord.Audio;
using Discord.WebSocket;
using Elementary.Dictionary;
using Microsoft.Extensions.DependencyInjection;
using NAudio.Wave;
using NLog;

namespace Elementary.Audio;

public class AudioManager
{
    private DiscordSocketClient _client;
    private IServiceProvider _services;
    private IAudioClient _audioClient;
    private AudioOutStream _audioStream;

    private SozaiAPI _sozaiAPI;
    private VoicevoxAPI _voicevoxAPI;
    private Ytdlp _ytdlp;

    private AudioConverter _audioConverter;

    private PlaybackQueue _playbackQueue;
    private EmojiDictionary _emojiDictionary;
    private DictionaryDB _dictionaryDB;
    private Mecab _mecab;
    // private AudioMixer _audioMixer;

    private ILogger _logger;

    public bool isConnected;

    public AudioManager(IServiceProvider services, DiscordSocketClient client, SozaiAPI sozaiApi,
        VoicevoxAPI voicevoxApi, EmojiDictionary emojiDictionary, DictionaryDB dictionaryDB, Mecab mecab, Ytdlp ytdlp)
    {
        _services = services;
        _client = client;
        _sozaiAPI = sozaiApi;
        _voicevoxAPI = voicevoxApi;
        _ytdlp = ytdlp;
        _audioConverter = new();
        _emojiDictionary = emojiDictionary;
        _dictionaryDB = dictionaryDB;
        _mecab = mecab;
        _logger = LogManager.GetCurrentClassLogger();
    }

    public async Task JoinChannel(IVoiceChannel channel)
    {
        _audioClient = await channel.ConnectAsync(true);
        // _audioClient.ClientDisconnected += async (exception) =>
        // {
        //     isConnected = false;
        //     await JoinChannel(channel);
        // };
        _audioClient.Disconnected += async (exception) =>
        {
            isConnected = false;
            await JoinChannel(channel);
        };
        
        _audioClient.StreamCreated += async (id, stream) =>
        {
            _logger.Log(LogLevel.Info, $"Stream created {id}");
            await Task.Delay(-1);
        };
        
        _audioStream = _audioClient.CreatePCMStream(AudioApplication.Music, 128 * 1024);

        _playbackQueue = _services.GetRequiredService<PlaybackQueue>();

        // _audioMixer = new AudioMixer(_audioStream);

        isConnected = true;
    }

    public async Task LeaveChannel()
    {
        await _audioClient.StopAsync();
        await _audioStream.FlushAsync();
        _audioStream.Dispose();
        _audioClient.Dispose();
    }

    public async Task PlayAudio(string path, float volume = 1.0f)
    {
        float maxVolume = 0.15f;
        
        await using var wave = _audioConverter.CreateStreamFromFilePath(path, volume * maxVolume);

        _logger.Log(LogLevel.Info, $"Playing {path} {wave.Length} bytes");

        // _audioMixer.AddStream(wave);
        await wave.CopyToAsync(_audioStream);
        // GC.Collect();

        // try
        // {
        //     await wave.CopyToAsync(_audioStream);
        // }
        // catch (Exception e)
        // {
        //     _logger.Log(LogLevel.Error, e);
        // }
        // finally
        // {
        //     await _audioStream.DisposeAsync();
        // }
    }
    
    public async Task PlayYoutube(string url)
    {
        var stream = await _ytdlp.GetStream(url);
        
        await using var wave = _audioConverter.CreateStreamFromStream(stream, 0.1f);
        // _audioMixer.AddStream(wave);
        await wave.CopyToAsync(_audioStream);
    }

    public async Task PlayText(string text)
    {
        if (text.Contains("```")) text = "コードブロック";
        if (text.StartsWith("!") || text.StartsWith(".")) return;
        text = Regex.Replace(text, @"https?://[\w/:%#\$&\?\(\)~\.=\+\-]+", "URL");
        
        text = Regex.Replace(text, @"<:[\w]+:[\d]+>", m => m.Value.Split(":")[1]); // <:emoji:123456789> -> emoji
        text = Regex.Replace(text, @"<a:[\w]+:[\d]+>", m => m.Value.Split(":")[1]); // <:emoji:123456789> -> emoji
        
        text = text.Replace("～", "ー");

        float volume = 0.1f;

        Stream? stream = await _sozaiAPI.GetAudioStream(text);
        if (stream == null)
        {
            text = _dictionaryDB.Replace(text);
        
            text = _emojiDictionary.Replace(text);
        
            text = await _mecab.ParseToKana(text);
            
            stream = await _voicevoxAPI.Speak(text);
            volume = 0.8f;
            if (stream == null) return;
        }
        
        await using var wave = _audioConverter.CreateStreamFromStream(stream, volume);
        // _audioMixer.AddStream(wave);
        await wave.CopyToAsync(_audioStream);
        // GC.Collect();
        
        // try
        // {
        //     await wave.CopyToAsync(_audioStream);
        // }
        // catch (Exception e)
        // {
        //     _logger.Log(LogLevel.Error, e);
        // }
        // finally
        // {
        //     await _audioStream.DisposeAsync();
        // }
    }

    public async Task StopAudio()
    {
        // _audioMixer.Stop();
        _playbackQueue.Flush();
        await _audioStream.FlushAsync();
    }
}