﻿using System.Net.Http.Json;
using System.Text;
using Microsoft.Extensions.Caching.Memory;
using NLog;

namespace Elementary.Audio;

public class VoicevoxAPI
{
    private UriBuilder _APIRootUrl;
    private HttpClient _client;
    private ILogger _logger;
    private MemoryCache _cache;

    public VoicevoxAPI()
    {
        _logger = LogManager.GetCurrentClassLogger();
        _cache = new MemoryCache(new MemoryCacheOptions());
    }

    public async Task Setup(string url)
    {
        _client = new HttpClient();
        Uri _url = new(url);

        _APIRootUrl = new UriBuilder($"{_url.Scheme}://{_url.Host}:{_url.Port}");
    }

    /// <summary>
    /// Returns a stream of the 24000Hz wave audio generated from the text
    /// </summary>
    /// <param name="text"></param>
    /// <param name="speaker"></param>
    /// <returns></returns>
    public async Task<Stream?> Speak(string text, string speaker = "47")
    {
        _logger.Info($"Requested TTS {text}");

        if (_cache.TryGetValue(text, out Stream? stream))
        {
            _logger.Info($"Cache hit {text}");
            return stream;
        }

        var query = await GetAudioQuery(text, speaker);
        if (query == null) return null;
        stream = await GetAudioStream(query, speaker);
        if (stream == null) return null;
        _cache.Set(text, stream);
        return stream;
    }

    internal async Task<string?> GetAudioQuery(string text, string speaker = "1")
    {
        UriBuilder builder = new(_APIRootUrl.Uri);
        builder.Path = "/audio_query";
        builder.Query = $"text={text}&speaker={speaker}";

            var response = await _client.PostAsync(builder.Uri, new StringContent(string.Empty));
            var stream = await response.Content.ReadAsStreamAsync();
            return await new StreamReader(stream).ReadToEndAsync();
    }

    internal async Task<Stream?> GetAudioStream(string query, string speaker = "1")
    {
        UriBuilder builder = new(_APIRootUrl.Uri);
        builder.Path = "/synthesis";
        builder.Query = $"speaker={speaker}";

        var response =
            await _client.PostAsync(builder.Uri, new StringContent(query, Encoding.UTF8, "application/json"));
        var stream = await response.Content.ReadAsStreamAsync();
        return stream;
    }
}