﻿using System.Diagnostics;
using System.Reflection;
using Discord;
using Discord.Commands;
using Elementary.Audio;
using Elementary.Dictionary;

namespace Elementary.Commands;

public class MessageCommands : ModuleBase<SocketCommandContext>
{
    private AudioManager _audioManager;
    private readonly IServiceProvider _serviceProvider;
    private readonly CommandService _commandService;
    private DictionaryDB _dictionaryDB;

    public MessageCommands(AudioManager audioManager, CommandService commandService, IServiceProvider serviceProvider,
        DictionaryDB dictionaryDB)
    {
        _audioManager = audioManager;
        _commandService = commandService;
        _serviceProvider = serviceProvider;
        _dictionaryDB = dictionaryDB;
    }

    [Command("help")]
    [Summary("Displays a list of commands.")]
    public async Task HelpAsync()
    {
        EmbedBuilder embedBuilder = new()
        {
            Title = "Elementary Commands",
            Description = "A list of commands for Elementary.",
            Color = Color.Blue
        };

        foreach (var module in _commandService.Modules)
        {
            string description = null;
            foreach (var command in module.Commands)
            {
                var result = await command.CheckPreconditionsAsync(Context, _serviceProvider);
                if (result.IsSuccess) description += $"!{command.Name} - {command.Summary}\n";
            }

            if (!string.IsNullOrWhiteSpace(description)) embedBuilder.AddField(module.Name, description);
        }

        await ReplyAsync(embed: embedBuilder.Build());
    }

    [Command("say")]
    [Summary("Echoes a message.")]
    public Task SayAsync([Remainder] [Summary("The text to echo")] string echo)
    {
        return ReplyAsync(echo);
    }

    [Command("join", RunMode = RunMode.Async)]
    [Summary("Joins a voice channel.")]
    public async Task JoinAsync(IVoiceChannel channel = null)
    {
        await _audioManager.JoinChannel(channel ?? (Context.User as IGuildUser)?.VoiceChannel);
    }

    [Command("leave", RunMode = RunMode.Async)]
    [Summary("Leaves a voice channel.")]
    public async Task LeaveAsync()
    {
        await _audioManager.LeaveChannel();
    }

    [Command("play", RunMode = RunMode.Async)]
    [Summary("Plays audio from a file.")]
    public async Task PlayAsync([Remainder] [Summary("The path to the file")] string path)
    {
        path = path.Trim('"');
        await _audioManager.PlayAudio(path);
    }

    [Command("stop", RunMode = RunMode.Async)]
    [Summary("Stops playing audio.")]
    public async Task StopAsync()
    {
        await _audioManager.StopAudio();
    }
    
    [Command("add", RunMode = RunMode.Async)]
    [Summary("Adds a word to the dictionary.")]
    public Task AddAsync( [Summary("From Surface")] string surface, [Summary("To Pronunciation")]string pronunciation)
    {
        _dictionaryDB.Add(surface, pronunciation);
        return ReplyAsync($"Added {surface} -> {pronunciation}");
    }
    
    [Command("del", RunMode = RunMode.Async)]
    [Summary("Deletes a word from the dictionary.")]
    public Task DeleteAsync([Summary("From Surface")] string surface)
    {
        _dictionaryDB.Remove(surface);
        return ReplyAsync($"Removed {surface}");
    }
    
    [Command("list", RunMode = RunMode.Async)]
    [Summary("Lists all words in the dictionary.")]
    public Task ListAsync()
    {
        var dict = _dictionaryDB.GetAll();
        string result = "```\n";
        foreach (var item in dict)
        {
            result += $"{item.Key} -> {item.Value}\n";
        }
        result += "```";
        return ReplyAsync(result);
    }
    
    [Command("ytdlp", RunMode = RunMode.Async)]
    [Summary("Play sound from Youtube URL.")]
    public async Task YtdlpAsync([Summary("Youtube URL")] string url)
    {
        await _audioManager.PlayYoutube(url);
    }
    
    [Command("memory", RunMode = RunMode.Async)]
    [Summary("Show memory usage.")]
    public async Task MemoryAsync()
    {
        var memory = Environment.WorkingSet;
        await ReplyAsync($"Memory: {memory / 1024 / 1024} MB");
    }
    
    [Command("restart", RunMode = RunMode.Async)]
    [Summary("Restart the bot.")]
    public async Task RestartAsync()
    {
        await ReplyAsync("Restarting...");
        await LeaveAsync();
        var assembly = Assembly.GetEntryAssembly();
        var path = assembly.Location;
        path = path.Replace(".dll", ".exe");
        Console.WriteLine(path);
        Process.Start(path);
        
        Environment.Exit(0);
        
    }
}