﻿using System.Collections.Concurrent;
using System.Reflection;
using Discord;
using Discord.Audio;
using Discord.Commands;
using Discord.WebSocket;
using Elementary.Audio;
using Elementary.Commands;
using Elementary.Dictionary;
using Elementary.Message;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NAudio.Wave;
using NLog;
using NLog.Extensions.Logging;
using Configuration = Elementary.Config.Configuration;
using ILogger = NLog.ILogger;
using LogLevel = NLog.LogLevel;

namespace Elementary;

public class Program
{
    private ILogger _logger;
    private DiscordSocketClient _client;
    private IServiceProvider _services;
    private CommandService _commands;
    private Handler _handler;
    private SozaiAPI _sozaiAPI;
    private VoicevoxAPI _voicevoxAPI;
    private EmojiDictionary _emojiDictionary;
    private DictionaryDB _dictionaryDB;
    private Mecab _mecab;
    private Ytdlp _ytdlp;

    public static void Main(string[] args)
    {
        while (true)
        {
            try
            {
                new Program().Setup().Wait();
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
            }
        }
    }

    private async Task Setup()
    {
        var configuration = new Configuration();
        _client = new DiscordSocketClient(new DiscordSocketConfig
        {
            GatewayIntents = GatewayIntents.AllUnprivileged | GatewayIntents.MessageContent
        });
        _services = new ServiceCollection()
            .AddLogging(builder =>
            {
                builder.ClearProviders();
                builder.SetMinimumLevel(Microsoft.Extensions.Logging.LogLevel.Trace);
                builder.AddNLog(configuration.ConfigurationRoot);
            })
            .AddSingleton(_client)
            .AddScoped<CommandService>()
            .AddTransient<Handler>()
            .AddScoped<AudioManager>()
            .AddScoped<PlaybackQueue>()
            .AddScoped<VoicevoxAPI>()
            .AddScoped<SozaiAPI>()
            .AddTransient<AudioConverter>()
            .AddScoped<MessageHandler>()
            .AddSingleton<EmojiDictionary>()
            .AddSingleton<DictionaryDB>()
            .AddScoped<Mecab>()
            .AddScoped<Ytdlp>()
            .BuildServiceProvider();

        _logger = LogManager.GetCurrentClassLogger();
        
        _commands = _services.GetRequiredService<CommandService>();

        _sozaiAPI = _services.GetRequiredService<SozaiAPI>();
        _voicevoxAPI = _services.GetRequiredService<VoicevoxAPI>();
        _ytdlp = _services.GetRequiredService<Ytdlp>();

        _emojiDictionary = _services.GetRequiredService<EmojiDictionary>();
        _dictionaryDB = _services.GetRequiredService<DictionaryDB>();
        _mecab = _services.GetRequiredService<Mecab>();

        _handler = new(_client, _commands, _services, _services.GetRequiredService<AudioManager>(),
            _services.GetRequiredService<MessageHandler>());

        _client.Log += Log;
        _commands.Log += Log;

        // Bass.Init(-1, 44100, DeviceInitFlags.NoSpeakerAssignment, IntPtr.Zero);

        await _handler.Setup();
        await _sozaiAPI.Setup(configuration.AppSettings.SozaiSettings.Url);
        await _voicevoxAPI.Setup(configuration.AppSettings.VoicevoxSettings.Url);
        await _emojiDictionary.Setup(configuration.AppSettings.EmojiSettings.DictionaryPath);
        await _dictionaryDB.Setup();
        await _mecab.Setup(configuration.AppSettings.MecabSettings.enabled,
            configuration.AppSettings.MecabSettings.dictionaryPath);
        await _client.LoginAsync(TokenType.Bot, configuration.AppSettings.DiscordSettings.Token);
        await _client.StartAsync();
        await _client.SetActivityAsync(new Game("!join"));

        // _client.Disconnected += async (exception) =>
        // {
        //     await _client.StopAsync();
        //     await _client.StartAsync();
        // };

        _client.UserVoiceStateUpdated += async (user, before, after) =>
        {
            if (user.Id != _client.CurrentUser.Id) return;
            if (before.VoiceChannel.Id == after.VoiceChannel.Id) return;

            if (after.VoiceChannel is not null)
            {
                _services.GetRequiredService<AudioManager>().JoinChannel(after.VoiceChannel);
            }
        };

        await Task.Delay(-1); //keep the program running
    }

    private Task Log(LogMessage msg)
    {
        _logger.Log(LogLevel.FromString(msg.Severity.ToString()), msg.Exception, $"{msg.Source}: {msg.Message}");
        return Task.CompletedTask;
    }
}